---------------------------------------------------------------------------------------------
-- QHBar Class
---------------------------------------------------------------------------------------------
QHBar = {};
QHBar.__index = QHBar;

function QHBar:new(name)

	-- Metatable
	local self = {};
     	setmetatable(self, QHBar);
	
	-- Position Variables
	self.x = 0;
	self.y = 0;
	self.align = "TOPLEFT";
	self.relative = "TOPLEFT";
	self.anchor = UIParent;
	self.width = 0;
	self.height = 0;
	self.moving = nil;
     	
     	-- Display Variables
	self.fading = nil;
	self.flashingfront = nil;
	self.flashingback = nil;

	-- Frame Variables
	self.name = name;
	self.frame = getglobal(self.name);
	if not self.frame then 
		self.frame = CreateFrame("Frame", self.name, UIParent, "QHBARTEMPLATE"); 
	end
	
	-- Left Label Variables
	self.labelleft = getglobal(QH:Concat(self.frame:GetName(), "_BarFront", "LabelLeft"));
	self.labelleftcolor = {r=1.0, g=1.0, b=1.0, a=1.0};
	self.labellefttext = "";

	-- Right Label Variables
	self.labelright = getglobal(QH:Concat(self.frame:GetName(), "_BarFront", "LabelRight"));
	self.labelrightcolor = {r=1.0, g=1.0, b=1.0, a=1.0};
	self.labelrighttext = "";
	
	-- Background Variables
	self.background = getglobal(QH:Concat(self.frame:GetName(), "_BarBack", "Background"));
	self.backgroundcolor = {r=0.0, g=0.0, b=0.0, a=0.0};
	
	-- Button Variables
	self.button = getglobal(QH:Concat(self.frame:GetName(), "_Button"));
	
	-- Front Bar Variables
	self.barfront = getglobal(QH:Concat(self.frame:GetName(), "_BarFront"));
	self.barfronttexture = getglobal(QH:Concat(self.frame:GetName(), "_BarFront", "Texture"));
	self.barfrontcolor = {r=0.0, g=1.0, b=0.0, a=1.0};
	self.barfrontvalue = 0;
	self.barmaxvalue = 100;
	
	-- Back Bar Variables
	self.barback = getglobal(QH:Concat(self.frame:GetName(), "_BarBack"));
	self.barbacktexture = getglobal(QH:Concat(self.frame:GetName(), "_BarBack", "Texture"));
	self.barbackcolor = {r=1.0, g=1.0, b=0.0, a=1.0};
	self.barbackvalue = 0;
	
	-- Spark Variables
	self.spark = getglobal(QH:Concat(self.frame:GetName(), "_BarFront", "Spark"));
	self.sparkalways = false;
	self.sparkwidthratio = 32/195;
	self.sparkheightratio = 3.2;
	
	-- Reference Variables
	self.obj = self;
	self.parent = nil;
	self.frame.obj = self;
	self.button.obj = self;
	self.barfront.obj = self;
	self.barback.obj = self;
	
	-- Call Update Function and Return
	self:Update();
	return self;
end

---------------------------------------------------------------------------------------------
-- QHBar Methods
---------------------------------------------------------------------------------------------
function QHBar:Clear()

	-- Display Variables
	self.fading = nil;
	self.flashingfront = nil;
	self.flashingback = nil;
	
	-- Left Label Variables
	self.labelleftcolor = {r=1.0, g=1.0, b=1.0, a=1.0};
	self.labellefttext = "";

	-- Right Label Variables
	self.labelrightcolor = {r=1.0, g=1.0, b=1.0, a=1.0};
	self.labelrighttext = "";

	-- Background Variables
	self.backgroundcolor = {r=0.0, g=0.0, b=0.0, a=0.0};

	-- Front Bar Variables
	self.barfrontcolor = {r=0.0, g=1.0, b=0.0, a=1.0};
	self.barfrontvalue = 0;
	self.barmaxvalue = 100;

	-- Back Bar Variables
	self.barbackcolor = {r=1.0, g=1.0, b=0.0, a=1.0};
	self.barbackvalue = 0;
	
	-- Call Update
	self:Update();
end

function QHBar:SetValue(value, barmaxvalue)

	self:SetValueFront(value, barmaxvalue);
	self:SetValueBack(value, barmaxvalue);
end

function QHBar:SetValueFront(value, barmaxvalue)
	self.barfrontvalue = value;
	self:SetMaxValue(barmaxvalue);
end

function QHBar:SetValueBack(value, barmaxvalue)
	self.barbackvalue = value;
	self:SetMaxValue(barmaxvalue);
end

function QHBar:SetMaxValue(barmaxvalue)
	self.barmaxvalue = barmaxvalue;
end

function QHBar:SetTextLeft(text)
	self.labellefttext = text;
end

function QHBar:SetTextRight(text)
	self.labelrighttext = text;
end

function QHBar:SetTextColorLeft(r, g, b, a)
	self.labelleftcolor.r = r;
	self.labelleftcolor.g = g;
	self.labelleftcolor.b = b;
	self.labelleftcolor.a = a;
end

function QHBar:SetTextColorRight(r, g, b, a)
	self.labelrightcolor.r = r;
	self.labelrightcolor.g = g;
	self.labelrightcolor.b = b;
	self.labelrightcolor.a = a;
end

function QHBar:SetPosition(x, y, align, relative, anchor)
	self.x = x;
	self.y = y;
	self.align = align;
	self.relative = relative;
	self.anchor = anchor;
end

function QHBar:SetSize(width, height)
	self:SetWidth(width);
	self:SetHeight(height);	
end

function QHBar:SetWidth(width)
	self.width = width;
end

function QHBar:SetHeight(height)
	self.height = height;
end

function QHBar:StartMoving(button)

	if self.obj.parent then
		self.obj.parent:StartMoving(button);
	elseif IsShiftKeyDown() then
		self.obj.moving = true;
		self.obj.frame:SetMovable(true);
		self.obj.frame:StartMoving();
	end
end

function QHBar:StopMoving()

	if self.obj.parent then
		self.obj.parent:StopMoving();
	else
		self.obj.frame:StopMovingOrSizing();
		align, anchor, relative, x, y = self.frame:GetPoint();
		if not anchor then anchor = UIParent; end
		self.obj:SetPosition(x, y, align, relative, anchor);
		self.obj.moving = nil;
	end
end
	
function QHBar:FadeIn()
	self.fading = -0.5;
end

function QHBar:FadeOut()
	self.fading = 0.5;
end

function QHBar:StartFlashingFront()
	self.flashingfront = 0;
end

function QHBar:StopFlashingFront()
	self.flashingfront = nil;
end

function QHBar:StartFlashingBack()
	self.flashingback = 0;
end

function QHBar:StopFlashingBack()
	self.flashingback = nil;
end

function QHBar:SetColorFront(r, g, b, a)
	self.barfrontcolor.r = r;
	self.barfrontcolor.g = g;
	self.barfrontcolor.b = b;
	self.barfrontcolor.a = a;
end

function QHBar:SetColorBack(r, g, b, a)
	self.barbackcolor.r = r;
	self.barbackcolor.g = g;
	self.barbackcolor.b = b;
	self.barbackcolor.a = a;
end

function QHBar:SetBackgroundColor(r, g, b, a)
	self.backgroundcolor.r = r;
	self.backgroundcolor.g = g;
	self.backgroundcolor.b = b;
	self.backgroundcolor.a = a;
end

function QHBar:StartMoving(button)

	if self.obj.parent then
		self.obj.parent:StartMoving(button);
	elseif IsShiftKeyDown() then
		self.obj.moving = true;
		self.obj.frame:SetMovable(true);
		self.obj.frame:StartMoving();
	end
end

function QHBar:StopMoving()

	if self.obj.parent then
		self.obj.parent:StopMoving();
	else
		self.obj.frame:StopMovingOrSizing();
		align, anchor, relative, x, y = self.frame:GetPoint();
		if not anchor then anchor = UIParent; end
		self.obj:SetPosition(x, y, align, relative, anchor);
		self.obj.moving = nil;
	end
end

function QHBar:Hide()

	if self.frame:IsVisible() then self.frame:Hide(); end
end

function QHBar:Show()
	
	if not self.frame:IsVisible() then self.frame:Show(); end
end

---------------------------------------------------------------------------------------------
-- QHBar Events
---------------------------------------------------------------------------------------------
function QHBar:UpdateAlpha(alpha)
	self.frame:SetAlpha(alpha);
end

function QHBar:Update()

	-- Secure Update
	self:SecureUpdate();

	-- Set Bar Positions
	self.spark:ClearAllPoints();
	if not self.barmaxvalue or self.barmaxvalue <= 0.0 then
	
		-- Zero Bars
		self.spark:Hide();
		self.barfront:SetValue(0);
		self.barback:SetValue(0);
		self.spark:SetPoint("CENTER", self.barfront, "LEFT", self.barfront:GetValue() * self.barfront:GetWidth(), -1);
	else
		-- Front Bar Value
		if self.barfrontvalue and self.barfrontvalue < self.barmaxvalue then
			self.barfront:SetValue(self.barfrontvalue/self.barmaxvalue);
		elseif self.barfrontvalue and self.barfrontvalue >= self.barmaxvalue then
			self.barfront:SetValue(self.barmaxvalue);
		else
			self.barfront:SetValue(0);
		end

		-- Back Bar Value
		if self.barbackvalue and self.barbackvalue < self.barmaxvalue then
			self.barback:SetValue(self.barbackvalue/self.barmaxvalue);
		elseif self.barbackvalue and self.barbackvalue >= self.barmaxvalue then
			self.barback:SetValue(self.barmaxvalue);
		else
			self.barback:SetValue(0);
		end

		-- Place Spark On Biggest Bar
		if self.barfrontvalue and self.barfrontvalue > 0.0 and (not self.barbackvalue or self.barfrontvalue >= self.barbackvalue) then

			-- Set Spark Position/Color
			self.spark:Show();
			self.spark:SetPoint("CENTER", self.barfront, "LEFT", self.barfront:GetValue() * self.barfront:GetWidth(), -1);
			if self.barfrontcolor then 
				self.spark:SetVertexColor(self.barfrontcolor.r, self.barfrontcolor.g, self.barfrontcolor.b, self.barfrontcolor.a); 
			end

		elseif self.barbackvalue and self.barbackvalue > 0.0 and (not self.barfrontvalue or self.barfrontvalue < self.barbackvalue) then

			-- Set Spark Position/Color
			self.spark:Show();
			self.spark:SetPoint("CENTER", self.barback, "LEFT", self.barback:GetValue() * self.barback:GetWidth(), -1);
			if self.barbackcolor then 
				self.spark:SetVertexColor(self.barbackcolor.r, self.barbackcolor.g, self.barbackcolor.b, self.barbackcolor.a); 
			end
			
		elseif self.sparkalways then

			-- Always Show Spark
			self.spark:Show();
			self.spark:SetPoint("CENTER", self.barfront, "LEFT", self.barfront:GetValue() * self.barfront:GetWidth(), -1);
			if self.barfrontcolor then
				self.spark:SetVertexColor(self.barfrontcolor.r, self.barfrontcolor.g, self.barfrontcolor.b, self.barfrontcolor.a); 
			end
		else
			-- Hide Spark
			self.spark:Hide();
		end
	end

	-- Set Left Text Label
	if not self.labellefttext then self.labellefttext = ""; end
	if not self.labelleftcolor then self.labelleftcolor = {r=0.0, g=0.0, b=0.0, a=0.0}; end
	self.labelleft:SetText(self.labellefttext);
	self.labelleft:SetTextColor(self.labelleftcolor.r, self.labelleftcolor.g, self.labelleftcolor.b, self.labelleftcolor.a);

	-- Set Right Text Label
	if not self.labelrighttext then self.labelrighttext = ""; end
	if not self.labelrightcolor then self.labelrightcolor = {r=0.0, g=0.0, b=0.0, a=0.0}; end
	self.labelright:SetText(self.labelrighttext);
	self.labelright:SetTextColor(self.labelrightcolor.r, self.labelrightcolor.g, self.labelrightcolor.b, self.labelrightcolor.a);
	self.labelright:SetText("");

	-- Set Front Bar Color
	if not self.barfrontcolor then self.barfrontcolor = {r=0.0, g=0.0, b=0.0, a=0.0}; end
	self.barfront:SetStatusBarColor(self.barfrontcolor.r, self.barfrontcolor.g, self.barfrontcolor.b, self.barfrontcolor.a);

	-- Set Back Bar Color
	if not self.barbackcolor then self.barbackcolor = {r=0.0, g=0.0, b=0.0, a=0.0}; end
	self.barback:SetStatusBarColor(self.barbackcolor.r, self.barbackcolor.g, self.barbackcolor.b, self.barbackcolor.a);

	-- Set Background Texture
	if not self.backgroundcolor then self.backgroundcolor = {r=0.0, g=0.0, b=0.0, a=0.0}; end
	self.background:SetTexture(self.backgroundcolor.r, self.backgroundcolor.g, self.backgroundcolor.b, self.backgroundcolor.a);

	-- Set Bar Fading
	if self.fading then

		-- Fade In
		if self.fading < 0.0 then
			self.fading = min(self.fading + 0.02, 0.0);
			self.frame:SetAlpha((0.5 + self.fading) / 0.5);

		-- Fade Out
		elseif self.fading > 0.0 then
			self.fading = max(self.fading - 0.02, 0.0);
			self.frame:SetAlpha((self.fading) / 0.5);

		-- Stop Fading
		else
			self.fading = nil;
		end
	end

	-- Set Front Bar Flashing
	if self.flashingfront then
		local ftime = self.flashingfront % 1.25;
		if ftime >= 0.5 then
			self.barfronttexture:SetAlpha(1);
		elseif ftime >= 0.25 then
			self.barfronttexture:SetAlpha(1 - (0.5 - ftime) / 0.25);
		else
			self.barfronttexture:SetAlpha(1 - (ftime / 0.25));
		end
		self.flashingfront = self.flashingfront + 0.02;
	end

	-- Set Back Bar Flashing
	if self.flashingback then
		local ftime = self.flashingback % 1.25;
		if ftime >= 0.5 then
			self.barbacktexture:SetAlpha(1);
			self.spark:SetAlpha(1);
		elseif ftime >= 0.25 then
			self.barbacktexture:SetAlpha(1 - (0.5 - ftime) / 0.25);
			self.spark:SetAlpha(1 - (0.5 - ftime) / 0.25);
		else
			self.barbacktexture:SetAlpha(1 - (ftime / 0.25));
			self.spark:SetAlpha(1 - (ftime / 0.25));
		end
		self.flashingback = self.flashingback + 0.02;
	end
end

function QHBar:SecureUpdate()

	-- Secure Code
	if not InCombatLockdown() then
		
		-- Verify Size
		if not self.width then self.width = 0; end
		if not self.height then self.height = 0; end
		if not self.sparkwidthratio then self.sparkwidthratio = 1; end
		if not self.sparkheightratio then self.sparkheightratio = 1; end
		
		-- Verify Position
		if not self.x then self.x = 0; end
		if not self.y then self.y = 0; end
		if not self.align then self.align = "TOPLEFT"; end

		-- Frame Size
		self.frame:SetWidth(self.width);
		self.frame:SetHeight(self.height);

		-- Button Size
		self.button:SetWidth(self.width);
		self.button:SetHeight(self.height);

		-- Front Bar Size
		self.barfront:SetWidth(self.width);
		self.barfront:SetHeight(self.height);

		-- Back Bar Size
		self.barback:SetWidth(self.width);
		self.barback:SetHeight(self.height);
		
		-- Spark Size
		self.spark:SetWidth(self.width * self.sparkwidthratio);
		self.spark:SetHeight(self.height * self.sparkheightratio);

		-- Frame Position
		if not self.moving then
			self.frame:ClearAllPoints();
			self.frame:SetPoint(self.align, self.anchor, self.relative, self.x, self.y);
		end
		
		-- Enable Movement
		self.frame:EnableMouse(true);
		self.frame:RegisterForDrag("LeftButton");
		self.frame:SetScript("OnDragStart", self.StartMoving);
		self.frame:SetScript("OnDragStop", self.StopMoving);
	end
end