---------------------------------------------------------------------------------------------
-- QHOptions Class
---------------------------------------------------------------------------------------------
QHOptions = {};
QHOptions.__index = QHOptions;

function QHOptions:new()

	-- Metatable
	local self = {};
     	setmetatable(self, QHOptions);

	-- Main Options
	QHOptions.panel = CreateFrame("Frame", "QHOptions_Frame", UIParent, "QHOPTIONSMAINTEMPLATE");
	QHOptions.panel.okay = self.SaveMainOptions;
	QHOptions.panel.cancel = self.SetupOptions;
	QHOptions.panel.label = QH_LOCALE_OPTIONSTAB;
	QHOptions_FrameLabel:SetText(QHOptions.panel.label);
	QHOptions_FrameSubLabel1:SetText(QH_LOCALE_SUBOPTIONS1);
	
	-- Player Bar Options
	QHOptions.playerpanel = CreateFrame("Frame", "QHOptions_Player_Bar", UIParent, "QHOPTIONSBARTEMPLATE");
	QHOptions.playerpanel.okay = self.SavePlayerOptions;
	QHOptions.playerpanel.cancel = self.SetupOptions;
	QHOptions.playerpanel.label = QH_LOCALE_PLAYERTAB;
	QHOptions_Player_BarLabel:SetText(QHOptions.playerpanel.label);
	
	-- Party Bar Options
	QHOptions.partypanel = CreateFrame("Frame", "QHOptions_Party_Bar", UIParent, "QHOPTIONSBARTEMPLATE");
	QHOptions.partypanel.okay = self.SavePartyOptions;
	QHOptions.partypanel.cancel = self.SetupOptions;
	QHOptions.partypanel.label = QH_LOCALE_PARTYTAB;
	QHOptions_Party_BarLabel:SetText(QHOptions.partypanel.label);
	
	-- Raid Bar Options
	QHOptions.raidpanel = CreateFrame("Frame", "QHOptions_Raid_Bar", UIParent, "QHOPTIONSBARTEMPLATE");
	QHOptions.raidpanel.okay = self.SaveRaidOptions;
	QHOptions.raidpanel.cancel = self.SetupOptions;
	QHOptions.raidpanel.label = QH_LOCALE_RAIDTAB;
	QHOptions_Raid_BarLabel:SetText(QHOptions.raidpanel.label);

	-- Register in the Interface Addon Options GUI
	QHOptions.panel.name = "Quick Heal 2";
	InterfaceOptions_AddCategory(QHOptions.panel);
	QHOptions.playerpanel.name = "Player Bar";
	QHOptions.playerpanel.parent = "Quick Heal 2";
	InterfaceOptions_AddCategory(QHOptions.playerpanel);
	QHOptions.partypanel.name = "Party Bars";
	QHOptions.partypanel.parent = "Quick Heal 2";
	InterfaceOptions_AddCategory(QHOptions.partypanel);
	QHOptions.raidpanel.name = "Raid Bars";
	QHOptions.raidpanel.parent = "Quick Heal 2";
	InterfaceOptions_AddCategory(QHOptions.raidpanel);
	
	return self;
end

function QHOptions:Update()

	QH.partyframe:Update(true);
	QH.raidframe:Update(true);
end

function QHOptions:SetupOptions()

	if QH then

		-- Main Options
		self:SetCheckButton(QHOptions_Frame_BarValues, QH_LOCALE_BARTEXT, QH:GetSetting("Bar_Label_Show"));
		self:SetCheckButton(QHOptions_Frame_ShowDebuffs, QH_LOCALE_SHOWDEBUFFS, QH:GetSetting("Decurse_Show"));
		self:SetCheckButton(QHOptions_Frame_RangeCheck, QH_LOCALE_RANGECHECK, QH:GetSetting("Bar_Range_Check"));
		self:SetCheckButton(QHOptions_Frame_BarBackground, QH_LOCALE_BARBACKDROP, QH:GetSetting("Bar_Background_Show"));
		self:SetCheckButton(QHOptions_Frame_AlwaysShowSpark, QH_LOCALE_ALWAYSSHOWSPARK, QH:GetSetting("Bar_Always_Show_Spark"));
		self:SetCheckButton(QHOptions_Frame_DisableTargetAdjust, QH_LOCALE_TARGETADJUST, QH:GetSetting("TargetFrame_Disable_Adjust"));
		self:SetCheckButton(QHOptions_Frame_RaidFrames, QH_LOCALE_RAIDENABLED, QH:GetSetting("Raid_Frame_Show"));
		self:SetCheckButton(QHOptions_Frame_HideParty, QH_LOCALE_PARTYHIDE, QH:GetSetting("Raid_Party_Hide"));
		self:SetCheckButton(QHOptions_Frame_RaidBackground, QH_LOCALE_RFBACKDROP, QH:GetSetting("Raid_Background_Show"));
		self:SetDropdown(QHOptions_Frame_RaidSort, QH_LOCALE_RAIDSORT, QH:GetSetting("Raid_Sort_By"), "CLASS", "GROUP");
		self:SetSlider(QHOptions_Frame_RaidRows, nil, QH_LOCALE_ROWCOUNT, 1, 40, 1, QH:GetSetting("Raid_Rows"));
		
		if QH:GetSetting("Raid_Background_Color") then
			self:SetSlider(QHOptions_Frame_RaidBackgroundTransparency, nil, QH_LOCALE_RFBACKDROPTRANS, 0, 100, 1, QH:GetSetting("Raid_Background_Color").a * 100);
		end
		
	
		-- Player Bar Options
		self:SetEditBox(QHOptions_Player_Bar_Left, QH_LOCALE_LEFT, QH:GetSetting("Player_Left_Click"));
		self:SetEditBox(QHOptions_Player_Bar_LeftShift, QH_LOCALE_LEFTSHIFT, QH:GetSetting("Player_Shift_Left_Click"));
		self:SetEditBox(QHOptions_Player_Bar_LeftCtrl, QH_LOCALE_LEFTCTRL, QH:GetSetting("Player_Ctrl_Left_Click"));
		self:SetEditBox(QHOptions_Player_Bar_LeftAlt, QH_LOCALE_LEFTALT, QH:GetSetting("Player_Alt_Left_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Middle, QH_LOCALE_MIDDLE, QH:GetSetting("Player_Middle_Click"));
		self:SetEditBox(QHOptions_Player_Bar_MiddleShift, QH_LOCALE_MIDDLESHIFT, QH:GetSetting("Player_Shift_Middle_Click"));
		self:SetEditBox(QHOptions_Player_Bar_MiddleCtrl, QH_LOCALE_MIDDLECTRL, QH:GetSetting("Player_Ctrl_Middle_Click"));
		self:SetEditBox(QHOptions_Player_Bar_MiddleAlt, QH_LOCALE_MIDDLEALT, QH:GetSetting("Player_Alt_Middle_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Right, QH_LOCALE_RIGHT, QH:GetSetting("Player_Right_Click"));
		self:SetEditBox(QHOptions_Player_Bar_RightShift, QH_LOCALE_RIGHTSHIFT, QH:GetSetting("Player_Shift_Right_Click"));
		self:SetEditBox(QHOptions_Player_Bar_RightCtrl, QH_LOCALE_RIGHTCTRL, QH:GetSetting("Player_Ctrl_Right_Click"));
		self:SetEditBox(QHOptions_Player_Bar_RightAlt, QH_LOCALE_RIGHTALT, QH:GetSetting("Player_Alt_Right_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Button4, QH_LOCALE_BUTTON4, QH:GetSetting("Player_Button4_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Button4Shift, QH_LOCALE_BUTTON4SHIFT, QH:GetSetting("Player_Shift_Button4_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Button4Ctrl, QH_LOCALE_BUTTON4CTRL, QH:GetSetting("Player_Ctrl_Button4_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Button4Alt, QH_LOCALE_BUTTON4ALT, QH:GetSetting("Player_Alt_Button4_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Button5, QH_LOCALE_BUTTON5, QH:GetSetting("Player_Button5_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Button5Shift, QH_LOCALE_BUTTON5SHIFT, QH:GetSetting("Player_Shift_Button5_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Button5Ctrl, QH_LOCALE_BUTTON5CTRL, QH:GetSetting("Player_Ctrl_Button5_Click"));
		self:SetEditBox(QHOptions_Player_Bar_Button5Alt, QH_LOCALE_BUTTON5ALT, QH:GetSetting("Player_Alt_Button5_Click"));
		self:SetSlider(QHOptions_Player_Bar_Width, nil, QH_LOCALE_BARWIDTH, 1, 100, 1, QH:GetSetting("Player_Bar_Width"));
		self:SetSlider(QHOptions_Player_Bar_Height, nil, QH_LOCALE_BARHEIGHT, 1, 40, 1, QH:GetSetting("Player_Bar_Height"));

		-- Party Bar Options
		self:SetEditBox(QHOptions_Party_Bar_Left, QH_LOCALE_LEFT, QH:GetSetting("Party_Left_Click"));
		self:SetEditBox(QHOptions_Party_Bar_LeftShift, QH_LOCALE_LEFTSHIFT, QH:GetSetting("Party_Shift_Left_Click"));
		self:SetEditBox(QHOptions_Party_Bar_LeftCtrl, QH_LOCALE_LEFTCTRL, QH:GetSetting("Party_Ctrl_Left_Click"));
		self:SetEditBox(QHOptions_Party_Bar_LeftAlt, QH_LOCALE_LEFTALT, QH:GetSetting("Party_Alt_Left_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Middle, QH_LOCALE_MIDDLE, QH:GetSetting("Party_Middle_Click"));
		self:SetEditBox(QHOptions_Party_Bar_MiddleShift, QH_LOCALE_MIDDLESHIFT, QH:GetSetting("Party_Shift_Middle_Click"));
		self:SetEditBox(QHOptions_Party_Bar_MiddleCtrl, QH_LOCALE_MIDDLECTRL, QH:GetSetting("Party_Ctrl_Middle_Click"));
		self:SetEditBox(QHOptions_Party_Bar_MiddleAlt, QH_LOCALE_MIDDLEALT, QH:GetSetting("Party_Alt_Middle_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Right, QH_LOCALE_RIGHT, QH:GetSetting("Party_Right_Click"));
		self:SetEditBox(QHOptions_Party_Bar_RightShift, QH_LOCALE_RIGHTSHIFT, QH:GetSetting("Party_Shift_Right_Click"));
		self:SetEditBox(QHOptions_Party_Bar_RightCtrl, QH_LOCALE_RIGHTCTRL, QH:GetSetting("Party_Ctrl_Right_Click"));
		self:SetEditBox(QHOptions_Party_Bar_RightAlt, QH_LOCALE_RIGHTALT, QH:GetSetting("Party_Alt_Right_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Button4, QH_LOCALE_BUTTON4, QH:GetSetting("Party_Button4_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Button4Shift, QH_LOCALE_BUTTON4SHIFT, QH:GetSetting("Party_Shift_Button4_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Button4Ctrl, QH_LOCALE_BUTTON4CTRL, QH:GetSetting("Party_Ctrl_Button4_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Button4Alt, QH_LOCALE_BUTTON4ALT, QH:GetSetting("Party_Alt_Button4_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Button5, QH_LOCALE_BUTTON5, QH:GetSetting("Party_Button5_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Button5Shift, QH_LOCALE_BUTTON5SHIFT, QH:GetSetting("Party_Shift_Button5_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Button5Ctrl, QH_LOCALE_BUTTON5CTRL, QH:GetSetting("Party_Ctrl_Button5_Click"));
		self:SetEditBox(QHOptions_Party_Bar_Button5Alt, QH_LOCALE_BUTTON5ALT, QH:GetSetting("Party_Alt_Button5_Click"));
		self:SetSlider(QHOptions_Party_Bar_Width, nil, QH_LOCALE_BARWIDTH, 1, 100, 1, QH:GetSetting("Party_Bar_Width"));
		self:SetSlider(QHOptions_Party_Bar_Height, nil, QH_LOCALE_BARHEIGHT, 1, 40, 1, QH:GetSetting("Party_Bar_Height"));

		-- Raid Bar Options
		self:SetEditBox(QHOptions_Raid_Bar_Left, QH_LOCALE_LEFT, QH:GetSetting("Raid_Left_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_LeftShift, QH_LOCALE_LEFTSHIFT, QH:GetSetting("Raid_Shift_Left_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_LeftCtrl, QH_LOCALE_LEFTCTRL, QH:GetSetting("Raid_Ctrl_Left_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_LeftAlt, QH_LOCALE_LEFTALT, QH:GetSetting("Raid_Alt_Left_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Middle, QH_LOCALE_MIDDLE, QH:GetSetting("Raid_Middle_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_MiddleShift, QH_LOCALE_MIDDLESHIFT, QH:GetSetting("Raid_Shift_Middle_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_MiddleCtrl, QH_LOCALE_MIDDLECTRL, QH:GetSetting("Raid_Ctrl_Middle_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_MiddleAlt, QH_LOCALE_MIDDLEALT, QH:GetSetting("Raid_Alt_Middle_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Right, QH_LOCALE_RIGHT, QH:GetSetting("Raid_Right_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_RightShift, QH_LOCALE_RIGHTSHIFT, QH:GetSetting("Raid_Shift_Right_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_RightCtrl, QH_LOCALE_RIGHTCTRL, QH:GetSetting("Raid_Ctrl_Right_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_RightAlt, QH_LOCALE_RIGHTALT, QH:GetSetting("Raid_Alt_Right_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Button4, QH_LOCALE_BUTTON4, QH:GetSetting("Raid_Button4_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Button4Shift, QH_LOCALE_BUTTON4SHIFT, QH:GetSetting("Raid_Shift_Button4_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Button4Ctrl, QH_LOCALE_BUTTON4CTRL, QH:GetSetting("Raid_Ctrl_Button4_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Button4Alt, QH_LOCALE_BUTTON4ALT, QH:GetSetting("Raid_Alt_Button4_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Button5, QH_LOCALE_BUTTON5, QH:GetSetting("Raid_Button5_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Button5Shift, QH_LOCALE_BUTTON5SHIFT, QH:GetSetting("Raid_Shift_Button5_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Button5Ctrl, QH_LOCALE_BUTTON5CTRL, QH:GetSetting("Raid_Ctrl_Button5_Click"));
		self:SetEditBox(QHOptions_Raid_Bar_Button5Alt, QH_LOCALE_BUTTON5ALT, QH:GetSetting("Raid_Alt_Button5_Click"));
		self:SetSlider(QHOptions_Raid_Bar_Width, nil, QH_LOCALE_BARWIDTH, 1, 50, 1, QH:GetSetting("Raid_Bar_Width"));
		self:SetSlider(QHOptions_Raid_Bar_Height, nil, QH_LOCALE_BARHEIGHT, 1, 40, 1, QH:GetSetting("Raid_Bar_Height"));
	end
end

function QHOptions:SaveMainOptions()
	
	if QH then
	
		QH:SetSetting("Bar_Label_Show", QHOptions_Frame_BarValues:GetChecked());
		QH:SetSetting("Decurse_Show", QHOptions_Frame_ShowDebuffs:GetChecked());
		QH:SetSetting("Bar_Range_Check", QHOptions_Frame_RangeCheck:GetChecked());
		QH:SetSetting("Bar_Background_Show", QHOptions_Frame_BarBackground:GetChecked());
		QH:SetSetting("Bar_Always_Show_Spark", QHOptions_Frame_AlwaysShowSpark:GetChecked());
		QH:SetSetting("TargetFrame_Disable_Adjust", QHOptions_Frame_DisableTargetAdjust:GetChecked());
		QH:SetSetting("Raid_Frame_Show", QHOptions_Frame_RaidFrames:GetChecked());
		QH:SetSetting("Raid_Party_Hide", QHOptions_Frame_HideParty:GetChecked());
		QH:SetSetting("Raid_Background_Show", QHOptions_Frame_RaidBackground:GetChecked());
		QH:SetSetting("Raid_Sort_By", UIDropDownMenu_GetSelectedValue(QHOptions_Frame_RaidSort));
		QH:SetSetting("Raid_Rows", QHOptions_Frame_RaidRows:GetValue());
		
		local raid_background_color = QH:GetSetting("Raid_Background_Color");
		raid_background_color.a = QHOptions_Frame_RaidBackgroundTransparency:GetValue() / 100;
		QH:SetSetting("Raid_Background_Color", raid_background_color);
		
		QHOptions:Update();
	end
end

function QHOptions:SavePlayerOptions()

	if QH then
	
		QH:SetSetting("Player_Left_Click", QHOptions_Player_Bar_Left:GetText());
		QH:SetSetting("Player_Shift_Left_Click", QHOptions_Player_Bar_LeftShift:GetText());
		QH:SetSetting("Player_Ctrl_Left_Click", QHOptions_Player_Bar_LeftCtrl:GetText());
		QH:SetSetting("Player_Alt_Left_Click", QHOptions_Player_Bar_LeftAlt:GetText());
		
		QH:SetSetting("Player_Middle_Click", QHOptions_Player_Bar_Middle:GetText());
		QH:SetSetting("Player_Shift_Middle_Click", QHOptions_Player_Bar_MiddleShift:GetText());
		QH:SetSetting("Player_Ctrl_Middle_Click", QHOptions_Player_Bar_MiddleCtrl:GetText());
		QH:SetSetting("Player_Alt_Middle_Click", QHOptions_Player_Bar_MiddleAlt:GetText());

		QH:SetSetting("Player_Right_Click", QHOptions_Player_Bar_Right:GetText());
		QH:SetSetting("Player_Shift_Right_Click", QHOptions_Player_Bar_RightShift:GetText());
		QH:SetSetting("Player_Ctrl_Right_Click", QHOptions_Player_Bar_RightCtrl:GetText());
		QH:SetSetting("Player_Alt_Right_Click", QHOptions_Player_Bar_RightAlt:GetText());
		
		QH:SetSetting("Player_Button4_Click", QHOptions_Player_Bar_Button4:GetText());
		QH:SetSetting("Player_Shift_Button4_Click", QHOptions_Player_Bar_Button4Shift:GetText());
		QH:SetSetting("Player_Ctrl_Button4_Click", QHOptions_Player_Bar_Button4Ctrl:GetText());
		QH:SetSetting("Player_Alt_Button4_Click", QHOptions_Player_Bar_Button4Alt:GetText());

		QH:SetSetting("Player_Button5_Click", QHOptions_Player_Bar_Button5:GetText());
		QH:SetSetting("Player_Shift_Button5_Click", QHOptions_Player_Bar_Button5Shift:GetText());
		QH:SetSetting("Player_Ctrl_Button5_Click", QHOptions_Player_Bar_Button5Ctrl:GetText());
		QH:SetSetting("Player_Alt_Button5_Click", QHOptions_Player_Bar_Button5Alt:GetText());
		
		QH:SetSetting("Player_Bar_Width", QHOptions_Player_Bar_Width:GetValue());
		QH:SetSetting("Player_Bar_Height", QHOptions_Player_Bar_Height:GetValue());
		
		QHOptions:Update();
	end
end

function QHOptions:SavePartyOptions()

	if QH then
	
		QH:SetSetting("Party_Left_Click", QHOptions_Party_Bar_Left:GetText());
		QH:SetSetting("Party_Shift_Left_Click", QHOptions_Party_Bar_LeftShift:GetText());
		QH:SetSetting("Party_Ctrl_Left_Click", QHOptions_Party_Bar_LeftCtrl:GetText());
		QH:SetSetting("Party_Alt_Left_Click", QHOptions_Party_Bar_LeftAlt:GetText());
		
		QH:SetSetting("Party_Middle_Click", QHOptions_Party_Bar_Middle:GetText());
		QH:SetSetting("Party_Shift_Middle_Click", QHOptions_Party_Bar_MiddleShift:GetText());
		QH:SetSetting("Party_Ctrl_Middle_Click", QHOptions_Party_Bar_MiddleCtrl:GetText());
		QH:SetSetting("Party_Alt_Middle_Click", QHOptions_Party_Bar_MiddleAlt:GetText());

		QH:SetSetting("Party_Right_Click", QHOptions_Party_Bar_Right:GetText());
		QH:SetSetting("Party_Shift_Right_Click", QHOptions_Party_Bar_RightShift:GetText());
		QH:SetSetting("Party_Ctrl_Right_Click", QHOptions_Party_Bar_RightCtrl:GetText());
		QH:SetSetting("Party_Alt_Right_Click", QHOptions_Party_Bar_RightAlt:GetText());
		
		QH:SetSetting("Party_Button4_Click", QHOptions_Party_Bar_Button4:GetText());
		QH:SetSetting("Party_Shift_Button4_Click", QHOptions_Party_Bar_Button4Shift:GetText());
		QH:SetSetting("Party_Ctrl_Button4_Click", QHOptions_Party_Bar_Button4Ctrl:GetText());
		QH:SetSetting("Party_Alt_Button4_Click", QHOptions_Party_Bar_Button4Alt:GetText());

		QH:SetSetting("Party_Button5_Click", QHOptions_Party_Bar_Button5:GetText());
		QH:SetSetting("Party_Shift_Button5_Click", QHOptions_Party_Bar_Button5Shift:GetText());
		QH:SetSetting("Party_Ctrl_Button5_Click", QHOptions_Party_Bar_Button5Ctrl:GetText());
		QH:SetSetting("Party_Alt_Button5_Click", QHOptions_Party_Bar_Button5Alt:GetText());
		
		QH:SetSetting("Party_Bar_Width", QHOptions_Party_Bar_Width:GetValue());
		QH:SetSetting("Party_Bar_Height", QHOptions_Party_Bar_Height:GetValue());
		
		QHOptions:Update();
	end
end

function QHOptions:SaveRaidOptions()

	if QH then
	
		QH:SetSetting("Raid_Left_Click", QHOptions_Raid_Bar_Left:GetText());
		QH:SetSetting("Raid_Shift_Left_Click", QHOptions_Raid_Bar_LeftShift:GetText());
		QH:SetSetting("Raid_Ctrl_Left_Click", QHOptions_Raid_Bar_LeftCtrl:GetText());
		QH:SetSetting("Raid_Alt_Left_Click", QHOptions_Raid_Bar_LeftAlt:GetText());
		
		QH:SetSetting("Raid_Middle_Click", QHOptions_Raid_Bar_Middle:GetText());
		QH:SetSetting("Raid_Shift_Middle_Click", QHOptions_Raid_Bar_MiddleShift:GetText());
		QH:SetSetting("Raid_Ctrl_Middle_Click", QHOptions_Raid_Bar_MiddleCtrl:GetText());
		QH:SetSetting("Raid_Alt_Middle_Click", QHOptions_Raid_Bar_MiddleAlt:GetText());

		QH:SetSetting("Raid_Right_Click", QHOptions_Raid_Bar_Right:GetText());
		QH:SetSetting("Raid_Shift_Right_Click", QHOptions_Raid_Bar_RightShift:GetText());
		QH:SetSetting("Raid_Ctrl_Right_Click", QHOptions_Raid_Bar_RightCtrl:GetText());
		QH:SetSetting("Raid_Alt_Right_Click", QHOptions_Raid_Bar_RightAlt:GetText());
		
		QH:SetSetting("Raid_Button4_Click", QHOptions_Raid_Bar_Button4:GetText());
		QH:SetSetting("Raid_Shift_Button4_Click", QHOptions_Raid_Bar_Button4Shift:GetText());
		QH:SetSetting("Raid_Ctrl_Button4_Click", QHOptions_Raid_Bar_Button4Ctrl:GetText());
		QH:SetSetting("Raid_Alt_Button4_Click", QHOptions_Raid_Bar_Button4Alt:GetText());

		QH:SetSetting("Raid_Button5_Click", QHOptions_Raid_Bar_Button5:GetText());
		QH:SetSetting("Raid_Shift_Button5_Click", QHOptions_Raid_Bar_Button5Shift:GetText());
		QH:SetSetting("Raid_Ctrl_Button5_Click", QHOptions_Raid_Bar_Button5Ctrl:GetText());
		QH:SetSetting("Raid_Alt_Button5_Click", QHOptions_Raid_Bar_Button5Alt:GetText());
		
		QH:SetSetting("Raid_Bar_Width", QHOptions_Raid_Bar_Width:GetValue());
		QH:SetSetting("Raid_Bar_Height", QHOptions_Raid_Bar_Height:GetValue());
		
		QHOptions:Update();
	end
end

function QHOptions:SetCheckButton(checkbutton, label, value)

	if checkbutton and QH then
	
		local fonstring = getglobal(QH:Concat(checkbutton:GetName(), "Text"));
		if label then
			checkbutton.label = label;
			fonstring:SetText(label);
		end
		if value then checkbutton:SetChecked(value); 
		else checkbutton:SetChecked(false); end
	end
end

function QHOptions:SetEditBox(editbox, label, value)

	if editbox and QH then
	
		local fontstring = getglobal(QH:Concat(editbox:GetName(), "Text"));
		if label then
			editbox.label = label;
			fontstring:SetText(label);
		end
		if value then editbox:SetText(value); 
		else editbox:SetText(""); end
		editbox:SetCursorPosition(0);
	end
end

function QHOptions:SetSlider(slider, width, label, min, max, step, value)

	if slider and QH then
	
		local fontstring = getglobal(QH:Concat(slider:GetName(), "Text"));
		if label then slider.label = label; end
		fontstring:SetText(QH:Concat(label," [", value, "]"));
		fontstring:SetTextColor(0.99, 0.82, 0);
		if width then slider:SetWidth(width); end
		if min and max then slider:SetMinMaxValues(min, max); end
		if step then slider:SetValueStep(step); end
		if value then slider:SetValue(value); end
	end
end

function QHOptions:SetDropdown(dropdown, label, value, ...)

	if dropdown and QH then
		
		local fontstring = getglobal(QH:Concat(dropdown:GetName(), "Label"));
		if label then 
			dropdown.label = label;
			fontstring:SetText(label); 
		end
		dropdown.arguments = {...};
		UIDropDownMenu_Initialize(dropdown, QHOptions.SetDropdownList);
		if value then UIDropDownMenu_SetSelectedValue(dropdown, value); end
	end
end

function QHOptions:SetDropdownList()
	
	local info = UIDropDownMenu_CreateInfo();
	for index,value in ipairs(self.arguments) do
		if(value) then
			info = {};
			info.text = value;
			info.value = value;
			info.owner = UIDROPDOWNMENU_INIT_MENU;
			info.func = QHOptions.DropdownOnClick;
			UIDropDownMenu_AddButton(info);
		end
	end
end

function QHOptions:DropdownOnClick()
	
	-- Set Dropdown Value	
	UIDropDownMenu_SetSelectedValue(self.owner, self.value);
end